@@ -17,13 +17,10 @@
 #' @param y Univariate time series of data values. May also be a 2 column data frame containing a date column.
 #' @param exo Matrix of exogenous variables. Can be used to specify regression effects or other seasonal effects like holidays, etc.
 #' @param freq Frequency of the data (1 (yearly), 4 (quarterly), 12 (monthly), 365.25/7 (weekly), 365.25 (daily)), default is NULL and will be automatically detected
-#' @param seasons The seasonal periods: i.e. c(365.25, 7 if yearly and weekly seasonality). Default is NULL and will be estimated via wavelet analysis.
-#' @param harmonics Vector for number of cycles per year for all of the seasons: i.e. c(unique(1:floor(12/2)), unique(1:floor(7/2))).
-#' @param cycle, The period for the longer-term cycle
-#' The default is NULL, which will use 1 harmonic per season (freq/seasons) if seasons is estimated via wavelet analysis for parsimony.
-#' Otherwise it will use c(1, 2, 4, 6, 12, 24, 365.25/7, 365.25/3.5, 365.25/2) to capture harmonics at
-#' yearly, semi-yearly, quarterly, every other month, monthly, twice per month, weekly, twice per week, and every other day to fully parameterize yearly, quarterly, monthly, and weekly
-#' but will keep only harmonics > 0 and <= freq/2. This is more parsimonious than using harmonics at 1:floor(seasons/2).
+#' @param seasons The seasonal periods: i.e. c(365.25, 7 if yearly and weekly seasonality). Default is NULL and will be estimated via wavelet analysis. 
+#' Can set to FALSE if want no seasonality
+#' @param cycle, The period for the longer-term cycle. Deafult is NULL and will be estimated via wavelet analysis.
+#' Can set to FALSE if want no cycle.
 #' @param decomp Decomposition model ("tend-cycle-seasonal", "trend-seasonal", "trend-cycle", "trend-noise")
 #' @param trend Trend specification ("random-walk", "random-walk-drift", "double-random-walk", "random-walk2"). The default is NULL which will choose the best of all specifications based on the maximum likelihood.
 #' "random-walk" is the random walk trend.
@@ -64,8 +61,6 @@
 #' @param verbose Logical whether to print messages or not
 #' @param unconstrained Logical whether to remove inequality constraints on the trend during estimation
 #' @param saturating_growth Force the growth rate to converge to 0 in the long term 
-#' @param full_seasonal_spectrum Whether to check for full spectrum of seasonality rather than weekly and yearly suggestion
-#' @param seasonal_spectrum, Vector of seasonal periods to check
 #' @import data.table
 #' @useDynLib autostsm, .registration=TRUE
 #' @return List of estimation values including a data table with coefficients, convergence code, frequency, decomposition, seasonality, cyclicality, and trend specification
@@ -84,18 +79,24 @@
 #' }
 #' @export
 stsm_estimate = function(y, exo = NULL, freq = NULL, decomp = NULL, trend = NULL, unconstrained = FALSE, saturating_growth = FALSE,
-                         multiplicative = NULL, par = NULL, seasons = NULL, harmonics = NULL, cycle = NULL,
+                         multiplicative = NULL, par = NULL, seasons = NULL, cycle = NULL,
                          det_obs = FALSE, det_trend = NULL, det_seas = FALSE, det_drift = FALSE, det_cycle = FALSE,
-                         sig_level = 0.01, optim_methods = c("BFGS", "NM", "CG", "SANN"), maxit = 10000, verbose = FALSE,
-                         full_seasonal_spectrum = FALSE, seasonal_spectrum = NULL){
-  #exo = freq = decomp = trend = multiplicative = par = seasons = harmonics = cycle = seasonal_spectrum = det_trend = NULL
-  #det_obs = det_seas = det_drift = det_cycle = unconstrained = saturating_growth = full_seasonal_spectrum = FALSE
+                         sig_level = 0.01, optim_methods = c("BFGS", "NM", "CG", "SANN"), maxit = 10000, verbose = FALSE){
+  #exo = freq = decomp = trend = multiplicative = par = seasons = cycle = det_trend = NULL
+  #det_obs = det_seas = det_drift = det_cycle = unconstrained = saturating_growth = FALSE
   #sig_level = 0.01
   #optim_methods = "BFGS"
   #maxit = 10000
   #verbose = TRUE
-  if(sig_level < 0.01 | sig_level > 0.1){
-    stop("sig_level must be between 0.01 and 0.1.")
+  #stsm_build_dates = autostsm:::stsm_build_dates
+  #stsm_init_pars = autostsm:::stsm_init_pars
+  #stsm_init_vals = autostsm:::stsm_init_vals
+  #stsm_constraints = autostsm:::stsm_constraints
+  #Rcpp::sourceCpp("src/kalmanfilter.cpp")
+  
+  #Argument checks
+  if(sig_level <= 0 | sig_level > 0.1){
+    stop("sig_level must be > 0 and <= 0.1.")
   }
   if(any(!optim_methods %in% c("NR", "BFGS", "BHHH", "SANN", "CG", "NM")) | length(optim_methods) < 1){
     stop("optim_methods must be a vector containing 'NR', 'BFGS', 'BHHH', 'SANN', 'CG', and/or 'NM'")
@@ -110,12 +111,48 @@ stsm_estimate = function(y, exo = NULL, freq = NULL, decomp = NULL, trend = NULL
       stop("decomp must be one of 'trend-noise', 'trend-cycle', 'trend-seasonal', 'trend-cycle-seasonal', or 'trend-seasonal-cycle'.")
     }
   }
+  if(!all(sapply(c(det_obs, det_seas, det_drift, det_cycle, unconstrained, saturating_growth, verbose), is.logical))){
+    stop("det_obs, det_seas, det_drift, det_cycle, unconstrained, saturating_growth must be logical.")
+  }
+  if(!is.null(seasons)){
+    if(!is.logical(seasons) | is.numeric(seasons)){
+      stop("seasons must be NULL, a numeric vector, or logical")
+    }
+  }
+  if(!is.null(cycle)){
+    if(!is.logical(cycle) | is.numeric(cycle)){
+      stop("cycle must be NULL, a numeric vector, or logical")
+    }
+  }
+  if(!is.null(exo)){
+    if(!is.data.frame(exo) | !is.data.table(exo)){
+      stop("exo must be a data frame or data table")
+    }
+  }
+  if(!(is.data.frame(y) | is.data.table(y) | stats::is.ts(y))){
+    stop("y must be a data frame or data table")
+  }else{
+    y = as.data.table(y)
+    if(ncol(y) != 2){
+      stop("y must have two columns")
+    }else if(!all(unlist(lapply(colnames(y), function(x){class(y[, c(x), with = FALSE][[1]])})) %in% 
+                  c("Date", "yearmon", "POSIXct", "POSIXt", "POSIXlt", "numeric"))){
+      stop("y must have a date and numeric column")
+    }
+  }
+  if(!is.null(freq)){
+    if(!is.numeric(freq)){
+      stop("freq must be numeric")
+    }
+  }
   
   #Get the frequency of the data
   y = stsm_detect_frequency(y, freq)
   y = stsm_build_dates(y)
   dates = y$dates
   freq = y$freq
+  freq_name = y$name
+  standard_freq = y$standard_freq
   y = y$data
   
   #Remove leading and trailing NAs
@@ -131,21 +168,6 @@ stsm_estimate = function(y, exo = NULL, freq = NULL, decomp = NULL, trend = NULL
     message("Detecting the appropriate decomposition...")
   }
   
-  #Detect trend type
-  if(is.null(trend)){
-    if(verbose == TRUE){
-      message("Detecting the trend type...")
-    }
-    trend = stsm_detect_trend(y, freq, sig_level = sig_level)
-    if(is.null(det_trend)){
-      det_trend = trend$det_trend
-    }
-    trend = trend$trend
-  }
-  if(is.null(det_trend)){
-    det_trend = FALSE
-  }
-  
   #Detect multiplicative model
   if(is.null(multiplicative)){
     if(verbose == TRUE){
@@ -161,19 +183,14 @@ stsm_estimate = function(y, exo = NULL, freq = NULL, decomp = NULL, trend = NULL
   prior = stsm_prior(y, freq)
   
   #Detect seasonality
-  if(is.null(seasons) & is.null(harmonics) & ifelse(!is.null(decomp), grepl("seasonal", decomp), TRUE)){
+  if(is.null(seasons) & ifelse(!is.null(decomp), grepl("seasonal", decomp), TRUE)){
     if(verbose == TRUE){
       message("Detecting seasonality...")
     }
-    seasons = stsm_detect_seasonality(y, freq, sig_level = 0.0001, prior, full = full_seasonal_spectrum, spectrum = seasonal_spectrum)
-    wavelet = TRUE
-  }else if(is.null(seasons) & !is.null(harmonics)){
-    seasons = freq/harmonics
-    wavelet = FALSE
-  }else{
+    seasons = stsm_detect_seasonality(y, freq, sig_level = 0.0001, prior, full_spectrum = (standard_freq == FALSE))
+  }
+  if(is.null(seasons) | all(seasons == F)){
     seasons = numeric(0)
-    harmonics = numeric(0)
-    wavelet = FALSE
   }
 
   #Detect cyclicality
@@ -182,10 +199,31 @@ stsm_estimate = function(y, exo = NULL, freq = NULL, decomp = NULL, trend = NULL
       message("Detecting cyclicallity...")
     }
     cycle = stsm_detect_cycle(y, freq, sig_level = 0.0001, prior)
-  }else{
+  }
+  if(is.null(cycle) | all(cycle == F)){
     cycle = numeric(0)
   }
   
+  #Reset the prior
+  if(length(seasons) > 0){
+    prior = stsm_prior(y, freq, seasons = seasons, cycle = cycle)
+  }
+  
+  #Detect trend type
+  if(is.null(trend)){
+    if(verbose == TRUE){
+      message("Detecting the trend type...")
+    }
+    trend = stsm_detect_trend(y, freq, sig_level = sig_level, prior = prior, seasons = seasons, cycle = cycle)
+    if(is.null(det_trend)){
+      det_trend = trend$det_trend
+    }
+    trend = trend$trend
+  }
+  if(is.null(det_trend)){
+    det_trend = FALSE
+  }
+  
   #Assign the decomposition
   if(is.null(decomp)){
     decomp = "trend"
@@ -212,33 +250,15 @@ stsm_estimate = function(y, exo = NULL, freq = NULL, decomp = NULL, trend = NULL
     decomp = "trend-noise"
   }
   
-  #Standardize
+  #Standardize and reset the prior
   mean = mean(y, na.rm = TRUE)
   sd = stats::sd(y, na.rm = TRUE)
   y = (y - mean)/sd
-  
-  #Get seasonal harmonics if not already set
-  if(is.null(harmonics)){
-    harmonics = stsm_harmonics(freq, decomp, seasons, wavelet)
-  }else{
-    harmonics = sort(unique(unlist(harmonics)))
-  }
-  
-  ##### Set up the initial values for estimation #####
-  #Use naive model as the prior
-  if(floor(freq) == 260){
-    #Ensure the weekday and daily model have the same starting values
-    y_use = stsm_detect_frequency(data.table(y = y, date = dates), 365.25)
-    y_use = stsm_build_dates(y_use)
-    prior = stsm_prior(y_use$data, y_use$freq, decomp, harmonics*7/5)
-    rm(y_use)
-  }else{
-    prior = stsm_prior(y, freq, decomp, harmonics)
-  }
+  prior = stsm_prior(y, freq, decomp, seasons, cycle)
   
   #Set the initial parameter values
   if(is.null(par)){
-    par = stsm_init_pars(y, freq, trend, cycle, decomp, harmonics, prior)
+    par = stsm_init_pars(y, freq, trend, cycle, decomp, seasons, prior)
   }
   
   #Set any fixed parameters
@@ -306,8 +326,7 @@ stsm_estimate = function(y, exo = NULL, freq = NULL, decomp = NULL, trend = NULL
   }
   
   ###### Initial values for the unobserved components #####
-  init = stsm_init_vals(y = y, par = par, freq = freq, trend = trend, 
-                        decomp = decomp, harmonics = harmonics, prior = prior)
+  init = stsm_init_vals(y, par, freq, trend, decomp, seasons, prior, cycle)
   par = c(par, P0 = 1)
   #Set uncertainty to be the rescaled variance of the time series
   
@@ -320,9 +339,9 @@ stsm_estimate = function(y, exo = NULL, freq = NULL, decomp = NULL, trend = NULL
   #Define the objective function
   objective = function(par, y, freq, decomp, trend, init){
     yt = matrix(y, nrow = 1)
-    sp = stsm_ssm(par = par, yt = yt, freq = freq, decomp = decomp, trend = trend, init = init)
-    ans = kalman_filter(B0 = matrix(sp$B0, ncol = 1), P0 = sp$P0, Dt = sp$Dt, At = sp$At, Ft = sp$Ft, Ht = sp$Ht, Qt = sp$Qt, Rt = sp$Rt,
-                        yt = yt, X = X, beta = sp$beta)
+    sp = stsm_ssm(par, yt, decomp, trend, init)
+    ans = kalman_filter(matrix(sp$B0, ncol = 1), sp$P0, sp$Dt, sp$At, sp$Ft, sp$Ht, sp$Qt, sp$Rt,
+                        yt, X, sp$beta)
     return(ans$loglik)
   }
   
@@ -352,9 +371,8 @@ stsm_estimate = function(y, exo = NULL, freq = NULL, decomp = NULL, trend = NULL
     abs(out$estimate[grepl("sig_", names(out$estimate)) | names(out$estimate) == "P0"])
   k = length(out$estimate) - 1
   n = length(y[!is.na(y)])
-  fit = suppressWarnings(data.table(trend = trend, freq = freq, 
+  fit = suppressWarnings(data.table(trend = trend, freq = freq, freq_name = freq_name, standard_freq = standard_freq,
                                seasons = paste(seasons, collapse = ", "),
-                               harmonics = ifelse(!is.null(harmonics), paste(harmonics, collapse = ", "), NA),
                                cycle = 2*pi/out$estimate[grepl("lambda", names(out$estimate))],
                                decomp = decomp, multiplicative = multiplicative, 
                                convergence = (out$code == 0), 
